use axum::routing::get;
use webbrowser;
//developed by Mgr. Jan Hrubos, GPLv3 License
#[tokio::main]
async fn main() {
	println!("----------------------------------------------------");
	println!("Starting Autoskola Free 8 server based on Axum Rust.");
	println!("----------------------------------------------------");
	if webbrowser::open("http://localhost:5500").is_ok() {
		println!("Default web browser opened! url: http://localhost:5500");
		println!("Press CTRL+C or CTRL+X to cancel the server.");
		println!("Mgr. Jan Hrubos, license GPLv3");
		println!("----------------------------------------------------");
	}else{
		println!("Default web browser not opened! Open it manually and run url: http://localhost:5500");
		println!("Press CTRL+C or CTRL+X to cancel the server.");
		println!("Mgr. Jan Hrubos, license GPLv3");
		println!("----------------------------------------------------");
	}
    let shared_state: AppState = AppState { num: 42 };
    let app = setup_routing(shared_state);
    let listener = tokio::net::TcpListener::bind("localhost:5500")
        .await
        .unwrap();
        
    
    let return1=axum::serve(listener, app).await.unwrap();
    println!("{:?}",return1);
    //webbrowser::open("http://localhost:5500/").expect("Failed to open URL in default browser!");
    

}

#[allow(dead_code)]
fn setup_routing(shared_state: AppState) -> axum::Router {
    axum::Router::new()
        .fallback(fallback)
        .nest_service("/assets", tower_http::services::ServeDir::new("assets"))
        .route("/", get(get_root))
        .route("/assets/m_sk8", get(get_msk8))
        .route("/assets/m_en8", get(get_men8))
        .route("/assets/en8", get(get_en8))
        .route("/assets/sk8", get(get_sk8))
        .with_state(shared_state) // only adds state to all prior routes
}


async fn fallback(uri: axum::http::Uri) -> impl axum::response::IntoResponse {
    (axum::http::StatusCode::NOT_FOUND, format!("No route defined in your elf/exe server program: {uri}"))
}

async fn get_root() -> axum::response::Html<&'static str> {
    include_str!("../assets/index.html").into()
}

async fn get_msk8() -> axum::response::Html<&'static str> {
    include_str!("../assets/m_sk8/index.html").into()
}

async fn get_men8() -> axum::response::Html<&'static str> {
    include_str!("../assets/m_en8/index.html").into()
}

async fn get_sk8() -> axum::response::Html<&'static str> {
    include_str!("../assets/sk8/index.html").into()
}

async fn get_en8() -> axum::response::Html<&'static str> {
    include_str!("../assets/en8/index.html").into()
}


#[derive(Clone, Debug, Copy)]
#[allow(dead_code)]
pub struct AppState {
    num: i32,
}
